#!/usr/bin/env perl

##
## Author......: See docs/credits.txt
## License.....: MIT
##

use strict;
use warnings;

use Digest::SHA qw (sha1_hex);

sub module_constraints { [[0, 255], [0, 255], [0, 55], [0, 55], [0, 55]] }

sub module_generate_hash
{
  my $word   = shift;
  my $salt   = shift;

  my $digest = sha1_hex ($salt . $word);

  my $hash = sprintf ("sha1\$%s\$%s", $salt, $digest);

  return $hash;
}

sub module_verify_hash
{
  my $line = shift;

  my ($hash, $word) = split (':', $line);

  return unless defined $hash;
  return unless defined $word;

  my @data = split ('\$', $hash);

  return unless scalar @data == 3;

  my $signature = shift @data;
  my $salt      = shift @data;
  my $checksum  = shift @data;

  return unless ($signature eq "sha1");
  return unless defined $checksum;

  my $word_packed = pack_if_HEX_notation ($word);

  my $new_hash = module_generate_hash ($word_packed, $salt);

  return ($new_hash, $word);
}

1;
